#------------------
# circuit generator
#------------------
# written by Bertrand 'BLam' Lamy (blam@tuxfamily.org)
# use this code to make Slune level faster!!!

import math
import Tkinter

import soya
import soya.editor


def is_null(v):
  return abs(v) < 0.01

def normalize(a, b, c, size = 1.0):
  s = size / math.sqrt(a * a + b * b + c * c)
  return (a * s, b * s, c * s)

def cross(v1, v2):
  return (  v1[1] * v2[2] - v1[2] * v2[1],
          - v1[0] * v2[2] + v1[2] * v2[0],
            v1[0] * v2[1] - v1[1] * v2[0])

def express(u, v, w):
  # u = kv + tw
  # return (k, t)
  d = v[0] * w[1] - v[1] * w[0]
  if(is_null(d)):
    d = v[0] * w[2] - v[2] * w[0]
    if(is_null(d)):
      d = v[2] * w[1] - v[1] * w[2]
      k = (u[2] * w[1] - u[1] * w[2]) / d
      t = (u[1] * v[2] - u[2] * v[1]) / d
    else:
      k = (u[0] * w[2] - u[2] * w[0]) / d
      t = (u[2] * v[0] - u[0] * v[2]) / d
  else:
    k = (u[0] * w[1] - u[1] * w[0]) / d
    t = (u[1] * v[0] - u[0] * v[1]) / d
  return (k, t)

class Quaternion:
  def __init__(self, x = 0.0, y = 0.0, z = 0.0, w = 0.0):
    self.x = x
    self.y = y
    self.z = z
    self.w = w
  def set_length(self, length):
    l = length / math.sqrt(self.x * self.x + self.y * self.y + self.z * self.z + self.w * self.w)
    self.x *= l
    self.y *= l
    self.z *= l
    self.w *= l
  def normalize(self):
    self.set_length(1.0)
  def slerp(self, q2, alpha):
    costheta = self.x * q2.x + self.y * q2.y + self.z * q2.z + self.w * q2.w
    if(costheta < 0.0):
      qa = (-self.x, -self.y, -self.z, -self.w)
      costheta = - costheta
    else:
      qa = (self.x, self.y, self.z, self.w)
    if(1.0 - costheta > 0.01):
      theta = math.acos(costheta)
      sintheta = math.sin(theta)
      scale1 = math.sin(theta * (1.0 - alpha)) / sintheta
      scale2 = math.sin(theta * alpha)         / sintheta
    else:
      scale1 = 1.0 - alpha
      scale2 = alpha
    q = Quaternion(scale1 * qa[0] + scale2 * q2.x,
                   scale1 * qa[1] + scale2 * q2.y,
                   scale1 * qa[2] + scale2 * q2.z,
                   scale1 * qa[3] + scale2 * q2.w)
    q.normalize()
    return q
  def to_matrix(self):
    q = Quaternion(self.x, self.y, self.z, self.w)
    q.normalize()
    xx = q.x * q.x
    yy = q.y * q.y
    zz = q.z * q.z
    xy = q.x * q.y
    xz = q.x * q.z
    yz = q.y * q.z
    wx = q.x * q.w
    wy = q.y * q.w
    wz = q.z * q.w
    return (1.0 - 2.0 * (yy + zz),       2.0 * (xy + wz),       2.0 * (xz - wy), 0.0,
                  2.0 * (xy - wz), 1.0 - 2.0 * (xx + zz),       2.0 * (yz + wx), 0.0,
                  2.0 * (xz + wy),       2.0 * (yz - wx), 1.0 - 2.0 * (xx + yy), 0.0,
            0.0, 0.0, 0.0, 1.0,
            1.0, 1.0, 1.0)
  def from_matrix(self, m):
    s = math.sqrt(abs(m[0] / m[16] + m[5] / m[17] + m[10] / m[18] + m[15]))
    self.x = - m[9] / m[18] + m[6] / m[17]
    self.y = - m[2] / m[16] + m[8] / m[18]
    self.z = - m[4] / m[17] + m[1] / m[16]
    self.w = s * 0.5
    if(s == 0.0):
      x = abs(self.x)
      y = abs(self.y)
      z = abs(self.z)
      w = abs(self.w)
      if(x >= y and x >= z and x >= w):
        self.x = 1.0
        self.y = 0.0
        self.z = 0.0
      else:
        if(y >= x and y >= z and y >= w):
          self.x = 0.0
          self.y = 1.0
          self.z = 0.0
        else:
          if(z >= x and z >= y and z >= w):
            self.x = 0.0
            self.y = 0.0
            self.z = 1.0
          else:
            print '*** arg'
    else:
      s = 0.5 / s
      self.x *= s
      self.y *= s
      self.z *= s
    self.normalize()
  def from_vector_and_angle(self, vector, angle):
    v = soya.Vector(vector.x, vector.y, vector.z)
    v.normalize()
    a = 0.5 * (angle * math.pi / 180.0)
    s = math.sin(a)
    self.x = v.x * s
    self.y = v.y * s
    self.z = v.z * s
    self.w = - math.cos(a)

    
def check_quad(quad):
  # compute normals of 2 triangles
  def triangle_normal(p1, p2, p3):
    v1 = soya.Vector()
    v1.set_start_end(p2, p1)
    v2 = soya.Vector()
    v2.set_start_end(p3, p1)
    return v1.cross_product(v2)
  v1 = triangle_normal(quad.vertices[0], quad.vertices[1], quad.vertices[2])
  v2 = triangle_normal(quad.vertices[1], quad.vertices[2], quad.vertices[3])
  EPSILON = 0.001
  if(abs(v1.x - v2.x) > EPSILON or abs(v1.y - v2.y) > EPSILON or abs(v1.z - v2.z) > EPSILON):
    w = quad.parent
    f = soya.Face(w, [quad.vertices[0], quad.vertices[1], quad.vertices[2]], quad.material)
    f.smooth_lit = quad.smooth_lit
    f = soya.Face(w, [quad.vertices[2], quad.vertices[3], quad.vertices[0]], quad.material)
    f.smooth_lit = quad.smooth_lit
    w.remove(quad)


class Circuit:
  def __init__(self):
    #--------------------
    # CIRCUIT GENERATOR
    #
    # circuit attribute:
    # - self           : list of tuples vectors (x, y, z[, incline]) that defines the circuit
    # - self.section   : list of 2D points (x, y) that represent a section of the circuit
    # - self.materials : list of materials to use associated with each couple of points
    #--------------------
    self.current_direction = (0.0, 0.0, 1.0)
    self.section = []
    self.matrices = []
    self.materials = []
    self.world = soya.World()
    self.matrix = self.world._matrix
    self.step = 1.0
    self.length = 50
    
  def to_faces(self, world):

    w = soya.World(world)
    points = []
    for p in self.section:
      points.append(soya.Point(w, p[0], p[1], 0.0))

    #>> compute new sections and create faces
    w._matrix = self.matrices[0]
    distance = 0.0
    for matrix in self.matrices[1:]:
      #>> compute distance between old and new matrix
      x = matrix[12] - w._matrix[12]
      y = matrix[13] - w._matrix[13]
      z = matrix[14] - w._matrix[14]
      d = math.sqrt(x * x + y * y + z * z)
      #>> compute old_points
      old_points = []
      for point in points:
        old_points.append(point % world)
      #>> compute new_points
      w._matrix = matrix
      new_points = []
      for point in points:
        new_points.append(point % world)
      #>> create faces
      i = 0
      distance2 = 0
      while(i < len(points) - 1):
        pt1 = old_points[i]
        pt2 = new_points[i]
        i += 1
        pt3 = new_points[i]
        pt4 = old_points[i]
        x = pt2.x - pt1.x
        y = pt2.y - pt1.y
        z = pt2.z - pt1.z
        d2 = math.sqrt(x * x + y * y + z * z)
        try:
          material = self.materials[i - 1]
          # (material, mode)
          if(material[1] == 0):
            # mode 0: use distance and allways all the texture height
            tex_x1 = distance
            tex_x2 = distance + d
            tex_y1 = 0.0
            tex_y2 = 1.0
          elif(material[1] == 1):
            # mode 1: use distance for x and y
            tex_x1 = distance
            tex_x2 = distance + d
            tex_y1 = distance2
            tex_y2 = distance2 + d2
          p1 = soya.Vertex(world, pt1.x, pt1.y, pt1.z, tex_x1, tex_y1)
          p2 = soya.Vertex(world, pt2.x, pt2.y, pt2.z, tex_x2, tex_y1)
          p3 = soya.Vertex(world, pt3.x, pt3.y, pt3.z, tex_x2, tex_y2)
          p4 = soya.Vertex(world, pt4.x, pt4.y, pt4.z, tex_x1, tex_y2)
          face = soya.Face(world, [p1, p2, p3, p4], material[0])
        except:
          p1 = soya.Vertex(world, pt1.x, pt1.y, pt1.z)
          p2 = soya.Vertex(world, pt2.x, pt2.y, pt2.z)
          p3 = soya.Vertex(world, pt3.x, pt3.y, pt3.z)
          p4 = soya.Vertex(world, pt4.x, pt4.y, pt4.z)
          face = soya.Face(world, [p1, p2, p3, p4], None)
        face.smooth_lit = 1
        distance2 += d2
        #>> cut quad if it is not plane
        check_quad(face)
        
      distance += d
      
    world.remove(w)

  def scale_section(self, scale_x, scale_y):
    old_section = self.section
    self.section = []
    for point in old_section:
      self.section.append((point[0] * scale_x, point[1] * scale_y))

  def add_section(self, nb_transition = 0):
    i = 0
    q1 = Quaternion()
    q2 = Quaternion()
    q1.from_matrix(self.matrix)
    q2.from_matrix(self.world._matrix)
    while(i < nb_transition):
      f = float(i) / nb_transition
      q = q1.slerp(q2, f)
      matrix = q.to_matrix()
      sx = (1.0 - f) * self.matrix[16] + f * self.world._matrix[16]
      sy = (1.0 - f) * self.matrix[17] + f * self.world._matrix[17]
      sz = (1.0 - f) * self.matrix[18] + f * self.world._matrix[18]
      m = (matrix[ 0] * sx, matrix[ 1] * sx, matrix[ 2] * sx, matrix[ 3],
           matrix[ 4] * sy, matrix[ 5] * sy, matrix[ 6] * sy, matrix[ 7],
           matrix[ 8] * sz, matrix[ 9] * sz, matrix[10] * sz, matrix[11],
           (1.0 - f) * self.matrix[12] + f * self.world._matrix[12], 
           (1.0 - f) * self.matrix[13] + f * self.world._matrix[13],
           (1.0 - f) * self.matrix[14] + f * self.world._matrix[14],
           1.0,
           sx, sy, sz)
      self.matrices.append(m)
      i += 1
    self.matrices.append(self.world._matrix)
    self.matrix = self.world._matrix

  def apply_transform(self, transform, time):
    if(transform[0] == 'step'):
      # ('step', time_start, new_step)
      if(time >= transform[1]):
        self.step = transform[2]
        return 0
      else:
        return 1
    elif(transform[0] == 'advance'):
      # ('advance', time_start, length)
      if((time - transform[1]) * self.step >= transform[2]):
        return 0
      else:
        return 1
    elif(transform[0] == 'turn lateral'):
      # ('turn lateral', time_start, radius, angle)
      angle_deg = math.asin(self.step * 0.5 / transform[2]) * 2.0 * 180.0 / math.pi
      if(angle_deg * (time - transform[1] + 1) >= abs(transform[3])):
        angle_deg = abs(transform[3]) - angle_deg * (time - transform[1])
        if(transform[3] < 0.0): angle_deg = -angle_deg
        self.world.turn_lateral(angle_deg)
        return 0
      else:
        if(transform[3] < 0.0): angle_deg = -angle_deg
        self.world.turn_lateral(angle_deg)
        return 1
    elif(transform[0] == 'rotate lateral'):
      # ('rotate lateral', time_start, radius, angle)
      d = self.step * 0.5 / transform[2]
      if(d > 1.0):
        angle_deg = abs(transform[3])
      else:
        angle_deg = math.asin(d) * 2.0 * 180.0 / math.pi
      if(angle_deg * (time - transform[1] + 1) >= abs(transform[3])):
        angle_deg = abs(transform[3]) - angle_deg * (time - transform[1])
        if(transform[3] < 0.0): angle_deg = -angle_deg
        self.world.rotate_lateral(angle_deg)
        return 0
      else:
        if(transform[3] < 0.0): angle_deg = -angle_deg
        self.world.rotate_lateral(angle_deg)
        return 1
    elif(transform[0] == 'turn vertical'):
      # ('turn vertical', time_start, radius, angle)
      d = self.step * 0.5 / transform[2]
      if(d > 1.0):
        angle_deg = abs(transform[3])
      else:
        angle_deg = math.asin(d) * 2.0 * 180.0 / math.pi
      if(angle_deg * (time - transform[1] + 1) >= abs(transform[3])):
        angle_deg = abs(transform[3]) - angle_deg * (time - transform[1])
        if(transform[3] < 0.0): angle_deg = -angle_deg
        self.world.turn_vertical(angle_deg)
        return 0
      else:
        if(transform[3] < 0.0): angle_deg = -angle_deg
        self.world.turn_vertical(angle_deg)
        return 1
    elif(transform[0] == 'rotate vertical'):
      # ('rotate vertical', time_start, radius, angle)
      d = self.step * 0.5 / transform[2]
      if(d > 1.0):
        angle_deg = abs(transform[3])
      else:
        angle_deg = math.asin(d) * 2.0 * 180.0 / math.pi
      if(angle_deg * (time - transform[1] + 1) >= abs(transform[3])):
        angle_deg = abs(transform[3]) - angle_deg * (time - transform[1])
        if(transform[3] < 0.0): angle_deg = -angle_deg
        self.world.rotate_vertical(angle_deg)
        return 0
      else:
        if(transform[3] < 0.0): angle_deg = -angle_deg
        self.world.rotate_vertical(angle_deg)
        return 1
    elif(transform[0] == 'turn incline'):
      # ('turn incline', time_start, length, angle)
      angle = float(transform[3]) / transform[2] * self.step
      if((time - transform[1]) * self.step >= transform[2]):
        a = float(transform[3]) - angle * (time - transform[1])
        self.world.turn_incline(a)
        return 0
      else:
        self.world.turn_incline(angle)
        return 1
    elif(transform[0] == 'scale'):
      # ('scale', time_start, length, scale_x, scale_y, scale_z)
      # TO DO
      pass
    elif(transform[0] == 'stabilize'):
      # ('stabilize', time_start, length, lateral, vertical, incline, scale_x, scale_y, scale_z)
      v = soya.Volume()
      v.turn_lateral (transform[3])
      v.turn_vertical(transform[4])
      v.turn_incline (transform[5])
      v.scale(transform[6], transform[7], transform[8])
      if((time - transform[1]) * self.step >= transform[2]):
        self.world._matrix = (v._matrix[ 0], v._matrix[ 1], v._matrix[ 2], v._matrix[ 3],
                              v._matrix[ 4], v._matrix[ 5], v._matrix[ 6], v._matrix[ 7],
                              v._matrix[ 8], v._matrix[ 9], v._matrix[10], v._matrix[11],
                              self.world._matrix[12], self.world._matrix[13], self.world._matrix[14], 1.0,
                              v._matrix[16], v._matrix[17], v._matrix[18])
        return 0
      else:
        q1 = Quaternion()
        q2 = Quaternion()
        q1.from_matrix(self.world._matrix)
        q2.from_matrix(v._matrix)
        f = 1.0 / (transform[2] / self.step - time)
        q = q1.slerp(q2, f)
        matrix = q.to_matrix()
        sx = (1.0 - f) * self.world._matrix[16] + f * v._matrix[16]
        sy = (1.0 - f) * self.world._matrix[17] + f * v._matrix[17]
        sz = (1.0 - f) * self.world._matrix[18] + f * v._matrix[18]
        sx = 1.0
        sy = 1.0
        sz = 1.0
        self.world._matrix = (matrix[ 0] * sx, matrix[ 1] * sx, matrix[ 2] * sx, matrix[ 3],
                              matrix[ 4] * sy, matrix[ 5] * sy, matrix[ 6] * sy, matrix[ 7],
                              matrix[ 8] * sz, matrix[ 9] * sz, matrix[10] * sz, matrix[11],
                              self.world._matrix[12], self.world._matrix[13], self.world._matrix[14], 1.0,
                              sx, sy, sz)
        return 1
    else:
      print 'WARNING : discarding unknwon transformation', tranform[0]
      return 0

  def apply_transforms(self, transforms, time_start = 0):
    i = time_start
    while(len(transforms) > 0):
      current_transforms = []
      synchronize = 0
      to_remove = []
      for transform in transforms:
        if(transform[0] == 'synchronize'):
          synchronize = 1
          to_remove.append(transform)
          break
        elif(transform[0] == 'start'):
          # ('start', lateral, vertical, incline, scale_x, scale_y, scale_z)
          self.world.turn_lateral (transform[1])
          self.world.turn_vertical(transform[2])
          self.world.turn_incline (transform[3])
          self.world.scale(transform[4], transform[5], transform[6])
          self.matrices.append(self.world._matrix)
          to_remove.append(transform)
        elif(transform[0] == 'end'):
          return
        else:
          if(transform[1] <= i):
            current_transforms.append(transform)
          else:
            break
      for transform in to_remove:
        transforms.remove(transform)
      if(synchronize == 1):
        for transform in current_transforms:
          transforms.remove(transform)
        self.apply_transforms(current_transforms, i)
        i = 0
      else:
        for transform in current_transforms:
          if(self.apply_transform(transform, i) == 0):
            transforms.remove(transform)
        self.world.advance(0.0, 0.0, self.step)
        self.matrices.append(self.world._matrix)
        i += 1

  def to_matrices(self):
    self.apply_transforms(self.transforms)

  def generate(self, world):
    self.to_matrices()
    self.to_faces(world)




#circuit_vector = (0.0, 0.2, 1.0)
#circuit_vectors = []

#circuit_vector = add_straight_road(circuit_vectors, circuit_vector, 2.0, 5)
#circuit_vector = add_turn_road(circuit_vectors, circuit_vector, 90.0, 4.0, 5)

c = Circuit()
#c.current_direction = (0.0, 0.2, 0.8)
#c.section = [(0.0, 0.0), (-1.0, 0.5), (1.0, 0.5), (0.0, 0.0)]

c.section = [( 0.0, -1.0),
             (-2.4,  0.5),
             (-2.0,  1.0),
             (-1.5,  1.0),
             (-1.1,  0.3),
             (-0.5,  0.0),
             ( 0.5,  0.0),
             ( 1.1,  0.3),
             ( 1.5,  1.0),
             ( 2.0,  1.0),
             ( 2.4,  0.5),
             ( 0.0, -1.0)
             ]

c.scale_section(0.6, 0.6)

#c.go_straight(8.0, 10)
#c.turn(4.0, -90.0, 10, 20.0, 0.2)
#c.go_straight(4.0, 4)
#c.turn(8.0, 90.0, 10, 40.0, 1.0)

c.transforms = [
  ('start', 5.0, 36.0, 0.0, 1.0, 1.0, 1.0),
  ('step', 0, 4.0),
  ('advance', 0, 3.0),
  ('synchronize', ),
  ('turn vertical', 0, 12.0, -35.0),
  ('synchronize', ),
  ('advance', 0, 8.0),
  ('synchronize', ),
  ('step', 0, 3.0),
  ('rotate lateral', 0, 6.0, -(360.0 + 170.0)),
  ('turn incline', 0, 10.0, 30.0),
  ('rotate vertical', 0, 1.0, -10.0),
  ('synchronize', ),
  ('step', 0, 4.0),
  ('stabilize', 0, 16.0, 190.0, 0.0, 0.0, 1.0, 1.0, 1.0),
  ('synchronize', ),
  ('step', 0, 1.0),
  ('rotate vertical', 0, 4.0, -195.0),
  ('synchronize', ),
  ('rotate vertical', 0, 2.5, -165.0),
  ('synchronize', ),
  ('step', 0, 4.0),
  ('rotate vertical', 0, 10.0, -30.0),
  ('synchronize', ),
  ('stabilize', 0, 8, 180.0, 0.0, 0.0, 1.0, 1.0, 1.0),
  ('step', 0, 3.0),
  ('synchronize', ),
  ('rotate vertical', 0, 8.0, 90.0),
  ('synchronize', ),
  ('step', 0, 2.0),
  ('turn lateral', 0, 3.0, 180.0),
  ('synchronize', ),
  ('step', 0, 4.0),
  ('advance', 0, 6.0),
  ('synchronize', ),
  ('step', 0, 2.0),
  ('rotate vertical', 0, 8.0, -90.0),
  ('synchronize', ),
  ('rotate lateral', 0, 6.0, 90.0),
  ('synchronize', ),
  ('turn vertical', 0, 12.0, 10.0),
  ('synchronize', ),
  ('turn vertical', 0, 6.0, -70.0),
  ('synchronize', ),
  ('turn vertical', 0, 5.0, 120.0),
  ('synchronize', ),
  ('turn vertical', 0, 6.0, -60.0),
  ('synchronize', ),
  ('turn vertical', 0, 5.0, -60.0),
  ('synchronize', ),
  ('turn vertical', 0, 4.0, 60.0),
  ('synchronize', ),
  ('turn vertical', 0, 4.0, -60.0),
  ('synchronize', ),
  ('turn vertical', 0, 4.0, 60.0),
  ('synchronize', ),
  ('advance', 0, 4.0),
  ('synchronize', ),
  ('turn incline', 0, 6.0, -50.0),
  ('rotate lateral', 0, 8.0, 80.0),
  ('synchronize', ),
  ('turn incline', 0, 6.0, 50.0),
  ('rotate lateral', 0, 8.0, 80.0),
  ('synchronize', ),
  ('step', 0, 2.0),
  ('turn incline', 0, 36.0, 360.0),
  ('synchronize', ),
  ('step', 0, 3.0),
  ('rotate lateral', 0, 8.0, 110.0),
  ('turn vertical', 0, 8.0, -5.0),
  ('synchronize', ),
  ('step', 0, 4.0),
  ('rotate lateral', 0, 35.0, 5.0),
  ('synchronize', ),
  ('step', 0, 2.0),
  ('rotate vertical', 0, 5.5, -340.0),
  ('synchronize', ),
  ('step', 0, 3.0),
  ('rotate vertical', 0, 14.0, -36.0),
  ]

soya.Material.PATH = "/home/jiba/src/slune/materials"

c.materials = [
  (soya.Material.get("trail1"), 0),
  (soya.Material.get("yellowroad1"), 0),
  (soya.Material.get("yellowroad1"), 0),
  (soya.Material.get("tole1"), 0),
  (soya.Material.get("tole1"), 0),
  (soya.Material.get("tole2"), 0),
  (soya.Material.get("tole1"), 0),
  (soya.Material.get("tole1"), 0),
  (soya.Material.get("yellowroad1"), 0),
  (soya.Material.get("yellowroad1"), 0),
  (soya.Material.get("trail1"), 0),
  ]
#c.world.turn_incline(0.0)
#c.matrices.append(c.world._matrix)
#c.matrix = c.world._matrix

#c.world.translate(4.0, 0.0, 4.0)
#c.world.turn_lateral(90.0)
#c.world.turn_incline(20.0)
#c.add_section(10)

#c.world.translate(4.0, 0.0, -4.0)
#c.world.turn_lateral(90.0)
#c.world.turn_incline(20.0)
#c.add_section(10)

#c.turn(90.0, 4.0, 5)
#c.go_straight(4.0, 2)
#c.vertical_loop(360.0, 4.0, 0.2, 20.0)

#print c

# looping
#i = 0.1
#while(i < 6.28):
#  circuit_vectors.append((0.2, math.sin(i), math.cos(i)))
#  i += 0.5


#circuit_points = [(0.0, -0.25), (-1.0, 0.25), (1.0, 0.25), (0.0, -0.25)]

print 'ok'

scene = soya.World()
c.generate(scene)
#c.to_faces(scene)

#f = soya.Face(scene, [soya.Vertex(scene, -20.0, -0.5, -20.0),
#                            soya.Vertex(scene, -20.0, -0.5,  20.0),
#                            soya.Vertex(scene,  20.0, -0.5,  20.0),
#                            soya.Vertex(scene,  20.0, -0.5, -20.0)
#                            ])

scene.scale(4.0, 4.0, 4.0)

scene.filename = "freestylepark-shape"
scene.save()

soya.editor.edit(scene)
Tkinter.mainloop()




#! /usr/bin/python -O
# -*- python -*-

# Slune
# Copyright (C) 2002-2003 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# launch this script to generate the level

import math, os, os.path
import copy

import Tkinter

import soya
import soya.model  as model
import soya.soya3d as soya3d
import soya.editor.main
import soya.land

import slune.level

import circuit_generator
import town_generator

soya.init()



c = soya3d.World.get('construct-urbancall')

w = soya3d.World()
w.shapify_args = ('tree', {})
w.add(c.children[0])
b = c.children[1]

for v in b:
    if v.shape:
        q = soya3d.World(w)
        q._matrix = v._matrix
        v.shape.to_faces(q)
        for f in q:
            for ve in f:
                ve.color = None
    else:
        w.add(v)


w.filename = "shape-urbancall"
w.save()

print '[ OK ]'

import sys
sys.exit(0)



scene = soya3d.World()

atm = soya3d.Atmosphere()
atm.ambient = (0.55, 0.53, 0.5, 1.0)
atm.bg_color  = (0.51, 0.51, 0.51, 1.0)
atm.skyplane = 1
atm.sky_color = (0.8, 0.6, 1.0, 1.0)
atm.fog_color = atm.bg_color
atm.fog_type    = 0
atm.fog_start   = 20.0
atm.fog_end     = 50.0
atm.fog         = 1
atm.fog_density = 0.001
atm.cloud = model.Material.get("clouds1")
scene.atmosphere = atm

light = soya3d.Light(scene)
light.directional = 1



model.Material.PATH = "/home/blam/prog/slune/materials"




def make_land():

    def set_material_area(material, x, y, w, h):
        j = 0
        while j < h:
            i = 0
            while i < w:
                land.set_texture(x + i, y + j, m)
                i += 1
            j += 1
    
    land = soya.land.Land()
    land.scale_factor = 1.5
    land.from_image(model.Image("/home/blam/data/urbancall_land.png"))
    land.map_size = 8
    land.multiply_height(8.0)
#    land.add_height(-10.0)
    land.set_texture_layer(model.Material.get("ground2"), 0.0,  150.0)
    land.set_texture_layer(model.Material.get("sand1"), 1.0,  150.0)
#    land.set_texture_layer(model.Material.get("grass1"), 1.0,  150.0)

    m = model.Material.get("grass1")
    set_material_area(m, 36, 87, 2, 9)
    set_material_area(m, 41, 87, 2, 9)
    set_material_area(m, 36, 100, 2, 11)
    set_material_area(m, 41, 100, 2, 11)

    import Image

    map = Image.open("/home/blam/data/urbancall_land_materials.png")
    m = model.Material.get("ground4")
    i = 0
    while(i < 129):
        j = 0
        while(j < 129):
            if map.getpixel((i, j)) == (255, 0, 0):
                land.set_texture(i, j, m)
            j = j + 1
        i = i + 1

    land.set_texture_layer(model.Material.get("grass1"), 3.0,  150.0)
    land.set_texture_layer_angle(model.Material.get("rocks3"), 0.0, 150.0, 40.0, 90.0)

    return land










m1 = model.Material.get('building1')
m2 = model.Material.get('house1win')
m3 = model.Material.get('ground1')
m4 = model.Material.get('building4')

m5 = model.Material.get('house3')
m6 = model.Material.get('house3b')
m7 = model.Material.get('bark1')

m8 = model.Material.get('inde_2')
m9 = model.Material.get('inde_4')
m10 = model.Material.get('inde_1')
m11 = model.Material.get('inde_3')

m12 = model.Material.get('building3')
m13 = model.Material.get('building5')
m14 = model.Material.get('building6')

m15 = model.Material.get('brick1')
m16 = model.Material.get('brick1win')
m17 = model.Material.get('tile2')

m18 = model.Material.get('planches1')
m19 = model.Material.get('window1')
m20 = model.Material.get('thatch1')

m21 = model.Material.get('taj1')

m22 = model.Material.get('column2a')
m23 = model.Material.get('column2b')

m24 = model.Material.get('house1')
m25 = model.Material.get('house1win')

m26 = model.Material.get('house2a')
m27 = model.Material.get('house2b')

m28 = model.Material.get('inde_6')

m_glass_1 = (m12, 'plak round', 0.5, 'complete', 1.0)
m_glass_2 = (m12, 'plak round', 0.5, 'complete', 2.0)
m_wall_1 = (m1, 'plak', 1.0, 'complete', 1.0)
m_wall_2 = (m1, 'plak', 1.0, 'complete', 2.0)
m_roof = (m3, 'plak', 1.0, 'plak', 1.0)
m_corniche = (m4, 'plak', 1.0, 'complete', 1.0)
m_house3_1 = (m5, 'plak round', 0.5, 'complete', 1.0)
m_house3_2 = (m6, 'plak round', 1.0, 'complete', 1.0)
m_bark = (m7, 'plak', 1.0, 'plak', 1.0)
m_indebloc_1 = (m8, 'plak round', 1.0, 'complete', 1.0)
m_indebloc_2 = (m8, 'plak round', 1.0, 'complete', 2.0)
m_indehole = (m11, 'plak round', 1.0, 'complete', 1.0)
m_indefrise = (m10, 'plak round', 1.0, 'complete', 1.0)
m_indefrise2 = (m8, 'plak round', 1.0, 'complete', 2.0)
m_build_1 = (m13, 'plak round', 0.5, 'complete', 1.0)
m_build_2 = (m13, 'plak round', 0.5, 'complete', 2.0)
m_glass_3 = (m14, 'plak round', 0.5, 'complete', 3.0)
m_glass_4 = (m14, 'plak round', 0.5, 'complete', 2.0)
m_ground_1 = (m3, 'plak', 1.0, 'complete', 1.0)
m_ground_2 = (m3, 'plak', 1.0, 'complete', 2.0)
m_ground_3 = (m3, 'plak', 1.0, 'complete', 3.0)
m_church = (m15, 'plak round', 1.0, 'complete', 2.0)
m_churchwin = (m16, 'plak round', 1.0, 'complete', 1.0)
m_churchtile = (m17, 'plak', 1.0, 'plak', 1.0)
m_wood = (m18, 'plak round', 1.0, 'complete', 2.0)
m_woodwin = (m19, 'plak round', 1.0, 'complete', 1.0)
m_minaret = (m21, 'plak round', 1.0, 'complete', 1.0)
m_old_1 = (m23, 'plak', 1.0, 'complete', 2.0)
m_old_2 = (m22, 'plak', 1.0, 'complete', 2.0)
m_old_3 = (m23, 'plak', 1.0, 'complete', 1.0)
m_tavern = (m24, 'plak round', 0.5, 'complete', 3.0)
m_tavernwin = (m25, 'plak round', 1.0, 'complete', 1.0)
m_white = (m27, 'plak round', 1.0, 'complete', 1.0)
m_whitewin = (m26, 'plak round', 1.0, 'complete', 1.0)
m_gold = (m28, 'plak', 1.0, 'plak', 1.0)


def set_style_tower1(h):
    h.section = [(0.0, 0.0), (0.0, 2.0), (0.0, 4.0), (0.25, 4.25),
                 (0.0, 4.5), (0.0, 8.5), (0.25, 8.75),
                 (0.0, 9.0), (0.0, 13.0), (0.25, 13.25)
                 ]
    h.materials = [ m_wall_2, m_glass_1, m_corniche, m_corniche,
                    m_glass_2, m_corniche, m_corniche,
                    m_glass_2, m_corniche
                    ]
    if len(h.points) <= 4:
        h.roof = ('flat', m3)

def set_style_tower2(h):
    h.section = [(0.0, 0.0), (0.0, 1.0), (0.0, 5.0),
                 (0.0, 6.0), (0.0, 10.0),
                 (0.0, 11.0), (0.0, 15.0),
                 (0.0, 16.0), (0.0, 18.0), (0.0, 19.0)
                 ]
    h.materials = [ m_wall_1, m_build_2, m_wall_1, m_build_2, m_wall_1, m_build_2, m_wall_1, m_build_1, m_wall_1
                    ]
    if len(h.points) <= 4:
        h.roof = ('flat', m3)

def set_style_tower_decal(h):
    h.section = [(0.0, 0.0), (0.0, 1.0), (0.0, 5.0), (0.0, 6.0),
                 (-1.0, 6.0), (-1.0, 7.0), (-1.0, 9.0), (-1.0, 10.0),
                 (-2.0, 10.0), (-2.0, 11.0), (-2.0, 13.0), (-2.0, 14.0)
                 ]
    h.materials = [ m_wall_1, m_build_2, m_wall_1, m_ground_1,
                    m_wall_1, m_build_1, m_wall_1, m_ground_1,
                    m_wall_1, m_build_1, m_wall_1
                    ]
    if len(h.points) <= 4:
        h.roof = ('flat', m3)

def set_style_tower3(h):
    h.section = [(0.0, 0.0), (0.0, 1.0), (0.0, 7.0),
                 (0.0, 8.0), (0.0, 14.0), (0.0, 15.0)
                 ]
    h.materials = [ m_wall_1, m_glass_3, m_wall_1, m_glass_3, m_wall_1
                    ]
    if len(h.points) <= 4:
        h.roof = ('flat', m3)
        
def set_style_tower_high_decal(h):
    h.section = [(0.0, 0.0), (0.0, 1.0), (0.0, 5.0), (0.0, 6.0), (0.0, 8.0), (0.0, 9.0),
                 (-3.0, 9.0), (-3.0, 10.0), (-3.0, 14.0), (-3.0, 15.0)
                 ]
    h.materials = [ m_wall_1, m_glass_4, m_wall_1, m_build_1, m_wall_1, m_ground_3,
                    m_wall_1, m_glass_4, m_wall_1
                    ]
    if len(h.points) <= 4:
        h.roof = ('flat', m3)

def set_style_yellow(h):
    h.section = [(0.0, 0.0), (0.0, 2.0), (0.2, 2.2), (0.2, 3.2),
                 (0.0, 3.4), (0.0, 5.4), (0.2, 5.6), (0.2, 6.6),
                 ]
    h.materials = [ m_house3_1, m_bark, m_house3_2, m_bark, m_house3_1, m_bark, m_house3_2 ]
    if len(h.points) == 4:
        h.roof = ('trapez', m7, m7, 3.0, 3.0, 0.5, 3.0)

def set_style_church(h):
    h.section = [(0.0, 0.0), (0.0, 2.0), (0.0, 3.0), (-1.0, 4.0),
                 (-1.0, 6.0), (-1.0, 7.0)
                 ]
    h.materials = [ m_church, m_churchwin, m_churchtile, m_church, m_churchwin
                    ]
    h.roof = ('pyramid', m17, 3.0, 0.5, 'complete', 3.0)

def set_style_wood(h):
    h.section = [(0.0, 0.0), (0.0, 2.0), (0.0, 3.0), 
                 ]
    h.materials = [ m_wood, m_woodwin
                    ]
    h.roof = ('pyramid', m20, 2.5, 0.5, 'complete', 2.5)

def set_style_indus(h):
    h.section = [(0.0, 0.0), (0.0, 2.0), (0.0, 3.0), (-0.5, 4.0), (-0.5, 5.0), (-0.5, 6.0), (-0.5, 7.0)
                 ]
    h.materials = [ m_indebloc_2, m_indehole, m_bark, m_indebloc_1, m_indehole, m_indefrise ]
#    h.section = [(0.0, 0.0), (0.0, 2.0), (0.0, 3.0), (-0.5, 4.0), (-0.5, 5.0), (-0.5, 6.0), (-1.0, 7.0)
#                 ]
#    h.materials = [ m_indebloc_2, m_indehole, m_indefrise, m_indebloc_1, m_indehole, m_indefrise ]
    if len(h.points) == 4:
        h.roof = ('trapez', m9, m8, 2.5, 0.0, 0.5, 3.0)

def set_style_minaret(h):
    h.section = [(0.0, 0.0), (0.0, 1.0), (0.0, 3.0), (0.0, 4.0), (0.0, 6.0), (1.0, 7.0)
                 ]
    h.materials = [ m_wall_1, m_minaret, m_wall_1, m_minaret, m_wall_1 ]
    h.roof = ('pyramid', m17, 3.0, 0.5, 'complete', 3.0)

def set_style_minaret2(h):
    h.section = [(0.0, 0.0), (0.0, 1.0), (0.0, 3.0), (0.0, 4.0), (0.0, 6.0), (1.0, 7.0),
                 (-1.0, 7.0), (-1.0, 8.0), (-1.0, 10.0), (0.0, 11.0)
                 ]
    h.materials = [ m_wall_1, m_minaret, m_wall_1, m_minaret, m_wall_1,
                    m_ground_2, m_wall_1, m_minaret, m_wall_1
                    ]
    h.roof = ('pyramid', m17, 3.0, 0.5, 'complete', 3.0)

def set_style_old(h):
    h.section = [(0.0, 0.0), (-0.5, 1.0), (-0.5, 3.0), (0.0, 4.0),
                 (-1.0, 5.0), (-3.0, 6.0)
                 ]
    h.materials = [ m_old_1, m_old_2, m_old_1,
                    m_gold, m_gold ]
    h.roof = ('pyramid', m28, 1.5, 1.0, 'complete', 1.0)
        
def set_style_tavern(h):
    h.section = [(0.0, 0.0), (0.0, 3.0), (0.0, 4.0), (-1.0, 5.0), (-1.0, 8.0), (-1.0, 9.0)
                 ]
    h.materials = [ m_tavern, m_tavernwin, m_bark, m_tavern, m_tavernwin
                    ]
    if len(h.points) == 4:
        h.roof = ('trapez', m7, m7, 2.5, 2.0, 0.5, 3.0)

def set_style_white(h):
    h.section = [(0.0, 0.0), (0.0, 1.0), (0.0, 3.0), (0.0, 4.0), (0.0, 6.0)
                 ]
    h.materials = [ m_white, m_whitewin, m_white, m_whitewin
                    ]
    if len(h.points) == 4:
        h.roof = ('trapez', m17, m27, 2.5, 0.0, 0.5, 3.0)

def set_style_tower_inde(h):
    h.section = [(0.0, 0.0), (0.0, 2.0), (-0.2, 2.2), (-0.2, 4.2), (-0.4, 4.4), (-0.4, 6.4), (-0.6, 6.6), (-0.6, 8.6)]
    h.materials = [ m_indefrise2, m_bark,
                    m_indefrise2, m_bark,
                    m_indefrise2, m_bark,
                    m_indefrise2
                    ]
    h.roof = ('trapez', m7, m8, 3.0, 0.0, 1.0, 2.0)
    

STYLES = [
    set_style_tower1,
    set_style_tower2,
    set_style_tower3,
    set_style_tower_decal,
    set_style_tower_high_decal,
    set_style_indus,
    set_style_yellow,
    set_style_church,
#    set_style_wood,
    set_style_minaret,
    set_style_minaret2,
    set_style_old,
    set_style_tavern,
    set_style_white,

    ]


STYLES_DICT = {

    (0, 0, 0) : set_style_old,
    (128, 128, 128) : set_style_indus,
    (0, 255, 0) : set_style_minaret,
    (0, 128, 0) : set_style_minaret2,
    (255, 255, 0) : set_style_yellow,
    (128, 0, 0) : set_style_tavern,
    (128, 128, 255) : set_style_tower_inde,

    (255, 0, 255) : set_style_tower1,
    (255, 0, 0) : set_style_tower2,
    (0, 0, 255) : set_style_tower3,

    (0, 0, 128) : set_style_tower_decal,
    (0, 255, 255) : set_style_tower_high_decal,

    (255, 255, 255) : set_style_tower3,

    }



land = make_land()
scene.shape = land

#buildings = soya3d.World()
buildings = town_generator.make_city("/home/blam/data/urbancall.png", "/home/blam/data/urbancall_styles.png", STYLES_DICT, land, scale = 1.5)

#buildings = town_generator.make_city("/home/blam/data/densecity2.png", "/home/blam/data/urbancall_styles.png", STYLES_DICT, scene.shape, scale = 1.0)
#buildings = town_generator.make_city("/home/blam/data/densecity.png", None, STYLES, scene.shape)


# some special buildings added
v = soya3d.World(buildings)
h = town_generator.House(town_generator.make_octogon(3.0, 1.0))
h.section = [(0.0, 0.0), (0.0, 0.25), (-0.25, 0.5)]
h.materials = [ m_old_3, m_old_3 ]
h.roof = ('pyramid', m28, 0.0, 1.0, 'plak', 1.0)
h.translate_points(150.0, 123.0)
h.draw(v)
h.set_on_land(land, v)
v.y = -1.5


# for concave buildings, must hide landscape specialy
ps = [(102.089859009, 58.6109313965), (116.946487427, 58.5390625), (116.91015625, 65.921875), (102.089859009, 65.921875)]
h = town_generator.House(ps)
h.set_on_land(land)
ps = [(102.08984375, 58.6007843018), (102.08984375, 73.4265670776), (109.383399963, 73.3726577759), (109.383399963, 58.6007843018)]
h = town_generator.House(ps)
h.set_on_land(land)
ps = [(117.059570312, 75.0983123779), (121.389450073, 75.0773468018), (121.389450073, 89.9033889771), (117.101951599, 89.9033889771)]
h = town_generator.House(ps)
h.set_on_land(land)
ps = [(114.123146057, 82.5742263794), (124.385452271, 82.6014404297), (124.385452271, 86.9033889771), (114.101951599, 86.9033889771)]
h = town_generator.House(ps)
h.set_on_land(land)
ps = [(138.28843689, 109.77797699), (161.723068237, 109.766601562), (161.723068237, 113.515975952), (138.28843689, 113.495613098)]
h = town_generator.House(ps)
h.set_on_land(land)
ps = [(138.293716431, 132.481170654), (161.705337524, 132.491348267), (161.705337524, 136.208969116), (138.293716431, 136.197601318)]
h = town_generator.House(ps)
h.set_on_land(land)
ps = [(91.602645874, 132.106262207), (110.903335571, 132.085296631), (110.903335571, 139.429168701), (91.5986480713, 139.429168701)]
h = town_generator.House(ps)
h.set_on_land(land)
ps = [(71.9325332642, 150.075393677), (71.8901519775, 164.903640747), (68.9972763062, 164.903640747), (69.0614547729, 150.075393677)]
h = town_generator.House(ps)
h.set_on_land(land)


# add palmtrees
v = soya3d.World(buildings)
v.shape = model.Shape.get('palmtree1')
v.scale(0.4, 0.4, 0.4)
v.set_xyz(55.0, 1.9, 133.5)

v = soya3d.World(buildings)
v.shape = model.Shape.get('palmtree1')
v.scale(0.4, 0.4, 0.4)
v.rotate_lateral(180.0)
v.set_xyz(62.5, 1.9, 133.5)

v = soya3d.World(buildings)
v.shape = model.Shape.get('palmtree1')
v.scale(0.4, 0.4, 0.4)
v.set_xyz(55.0, 1.9, 138.5)

v = soya3d.World(buildings)
v.shape = model.Shape.get('palmtree1')
v.scale(0.4, 0.4, 0.4)
v.rotate_lateral(180.0)
v.set_xyz(62.5, 1.9, 138.5)

v = soya3d.World(buildings)
v.shape = model.Shape.get('palmtree1')
v.scale(0.4, 0.4, 0.4)
v.set_xyz(55.0, 1.9, 153.5)

v = soya3d.World(buildings)
v.shape = model.Shape.get('palmtree1')
v.scale(0.4, 0.4, 0.4)
v.rotate_lateral(180.0)
v.set_xyz(62.5, 1.9, 153.5)

v = soya3d.World(buildings)
v.shape = model.Shape.get('palmtree1')
v.scale(0.4, 0.4, 0.4)
v.set_xyz(55.0, 1.9, 161.0)

v = soya3d.World(buildings)
v.shape = model.Shape.get('palmtree1')
v.scale(0.4, 0.4, 0.4)
v.rotate_lateral(180.0)
v.set_xyz(62.5, 1.9, 161.0)

v = soya3d.World(buildings)
v.shape = model.Shape.get('shape-urbancall-add')






#for v in buildings.children:
#    if isinstance(v, soya3d.World):
#        if not v.shape:
#            v.shape = v.shapify()
#            v.children = []
##         if v.shape:
##             v.translate(0.0, -1.5, 0.0)
##             v.shape.to_faces(v)
##             for f in v:
##                 for ve in f:
##                     ve.color = None
##             v.shape = None
#scene.add(buildings)


v = soya3d.Volume(scene)
#v.shape = buildings.shapify()
#v.y = 1.5
v.shape = model.Shape.get('shape-urbancall')



scene.filename = "world-urbancall"
scene.save()


print '[ OK ]'

soya.editor.edit(scene)
Tkinter.mainloop()





#! /usr/bin/python -O
# -*- python -*-

# Slune
# Copyright (C) 2004 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# launch this script to generate the level

import math, os, os.path
import copy

import soya
import soya.editor.main

import slune.globdef


sand0 = soya.Material()
sand0.texture = soya.Image.get("sand0.png")
sand0.filename = "sand0"
sand0.save()

sand2 = soya.Material()
sand2.texture = soya.Image.get("sand2.png")
sand2.separate_specular = 1
sand2.specular = (1.0, 1.0, 0.7, 1.0)
sand2.shininess = 10.0
sand2.filename = "sand2"
sand2.save()


w = soya.World()
atm = soya.SkyAtmosphere()
atm.ambient = (0.4, 0.3, 0.2, 1.0)
#atm.bg_color = atm.fog_color = (0.5, 0.4, 0.2, 1.0) # pour la tempête ?
atm.bg_color = atm.fog_color = (0.7, 0.6, 0.7, 1.0)
atm.skyplane = 1
atm.sky_color = (0.3, 0.7, 1.0, 1.0)
atm.fog_type    = 0
atm.fog_start   = 20.0
atm.fog_end     = 70.0
atm.fog         = 1
atm.fog_density = 0.001
atm.cloud = soya.Material.get("clouds1")
w.atmosphere = atm

sun = soya.Light(w)
sun.directional = 1
sun.look_at(soya.Vector(w, -0.4, -1.0, -0.3))
sun.diffuse = 0.8, 0.7, 0.4, 1.0

land = soya.Land(w)
land.from_image(soya.Image.get("map-desert.png"))
land.map_size = 8
land.scale_factor = 1.5
land.multiply_height(30.0)
land.set_material_layer(soya.Material.get("sand0"),  0.0,  6.0)
land.set_material_layer(soya.Material.get("sand1"), 5.0,  14.0)
land.set_material_layer(soya.Material.get("sand2"),  13.0, 22.0)
land.set_material_layer(soya.Material.get("rocks1"),  21.0, 50.0)

import Image # PIL

circuit = Image.open("/home/jiba/src/slune/images/map-desert-grass.png")
m = soya.Material.get("grass1")
i = 0
while(i < 257):
  j = 0
  while(j < 257):
    if circuit.getpixel((i, j)) == 255:
      land.set_material(i, j, m)
    j += 1
  i += 1


ww = soya.World()

v = soya.Volume(w)



w.filename = "world-desertic-sorry"
w.save()



print '[ OK ]'



# Slune
# Copyright (C) 2004 Jean-Baptiste LAMY
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from __future__ import generators

from slune.fight import *
import soya.opengl as soyaopengl
import slune.globdef as globdef, slune.sound as sound

def init_arena_game(level, start_pos):
  random.shuffle(start_pos)
  level.start_pos = start_pos
  
  # Copy them in level, since the different player may have different config.
  level.arena_nb_frags     = globdef.RACE_NB_LAPS
  level.arena_nb_opponents = globdef.RACE_NB_OPPONENTS
  
  slune_character.PlayerCharacter._old_die = slune_character.PlayerCharacter.die
  slune_character.PlayerCharacter.die = _die
  
  slune_character.Character._old_die = slune_character.Character.die
  slune_character.Character.die = _die
  
def init_arena_opponent(level, character, frags_label = 1, minimap = 1):
  pos = level.start_pos.pop(0)
  if isinstance(pos, tuple):
    character.move   (pos[0])
    character.look_at(pos[1])
  else:
    character.move   (pos)
    
  level.start_pos.append(pos)
  
  character.nb_frags = 0
  
  if character.played and character.player.active:
    from py2play.idler import IDLER
    if frags_label: IDLER.no_blackbands_group.insert(0, FragsLabel(character))
    if minimap:     IDLER.no_blackbands_group.insert(0, MiniMap(character.level))
    
def start_arena_game(level):
  for i in range(level.arena_nb_opponents - len(level.characters)):
    character = Opponent()
    character.set_level(level, 1)
    init_arena_opponent(level, character)
    
  #for character in level.characters:
  #  if isinstance(character, slune_character.Competitor): init_arena_opponent(character)
  
def end_arena_game(level):
  for widget in soya.root_widget.children:
    if isinstance(widget, FragsLabel):
      soya.root_widget.remove(widget)
      break

  slune_character.PlayerCharacter.die = slune_character.PlayerCharacter._old_die
  slune_character.Character      .die = slune_character.Character      ._old_die

        

def _die(character):
  if not hasattr(character.level, "start_pos"):
    if isinstance(character, slune_character.PlayerCharacter):
      slune_character.PlayerCharacter._old_die(character)
    else:
      slune_character.Character      ._old_die(character)
    return
    
  if len(slune_character.explosions) < 4: # Limit the number of simultaneous explosions
    explode = slune_character.Explosion(character.parent)
    explode.move(character)
    explode.life = 60
    explode.particle_width = explode.particle_height = 0.7

    sound.play("explose-3.wav", explode)

  if hasattr(character.pushed_by, "nb_frags"):
    from py2play.idler  import IDLER
    from py2play.player import CURRENT_PLAYER
    IDLER.message(_("%s fragged %s!") % (character.pushed_by.displayname(), character.displayname()))
    
    character.pushed_by.nb_frags += 1
    if character.pushed_by.nb_frags >= character.level.arena_nb_frags: # Pusher has won !
      characters = character.level.characters[:]
      characters.sort(lambda a, b: cmp(b.nb_frags, a.nb_frags))
      recapitulatif = "\n".join(["%s : %s frags" % (c.displayname(), c.nb_frags) for c in characters])

      if character.pushed_by is CURRENT_PLAYER.character:
        IDLER.level_completed(CURRENT_PLAYER.character, 1, _("You have won!") + "\n" + recapitulatif)
      else:
        IDLER.level_completed(CURRENT_PLAYER.character, 0, _("__gameover__") + "\n" + recapitulatif)

    #print _("%s fragged %s!") % (character.pushed_by.displayname(), character.displayname())
    #for c in character.level.characters:
    #  print c.displayname(), c.nb_frags
    #print
    
    character.pushed_by = None # Avoid multiple frags
    
  pos = character.level.start_pos.pop(0)
  if isinstance(pos, tuple):
    character.teleport(pos[0].x, pos[0].y, pos[0].z)
    character.look_at(pos[1])
  else:
    character.teleport(pos.x, pos.y, pos.z)
  
  character.level.start_pos.append(pos)
  
  character.life = 2.0


class FragsLabel(widget.Label):
  def __init__(self, character):
    self.character = character
    self.format = "Frags : %s"
    widget.Label.__init__(self)
    
    self.left = 10
    self.width  = 400
    self.height = 100
    self.old_nb_frags = -1
    
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    self.top = parent_top + soya.root_widget.height - 43
    
  def widget_begin_round(self):
    if self.old_nb_frags != self.character.nb_frags:
      self.text = self.format % self.character.nb_frags
      self.old_nb_frags = self.character.nb_frags


      
class Opponent(Fighter, slune_character.Competitor):
  die = _die
  
  def choose_target(self):
    best      = None
    best_dist = 10000.0
    for level in self.get_root():
      if isinstance(level, py2p_level.Level):
        for character in level.characters:
          if (not character is self):
            dist = character.distance_to(self)
            if dist < best_dist:
              best      = character
              best_dist = dist
    return best
  








class MiniMap(widget.Widget):
  def __init__(self, level):
    self.level = level
    self.point = Point()
    
    self.top = 10
    self.width  = 200
    self.height = 200
    self.left = soya.root_widget.width - self.width - 40
    
    flags = level.start_pos
    min_x = min_y =  10e1000
    max_x = max_y = -10e1000
    for flag in flags:
      if flag.x < min_x: min_x = flag.x
      if flag.x > max_x: max_x = flag.x
      if flag.z < min_y: min_y = flag.z
      if flag.z > max_y: max_y = flag.z
    f_x = self.width  / (max_x - min_x)
    f_y = self.height / (max_y - min_y)
    if f_x < f_y:
      f = f_x
      left = self.left
    else:
      f = f_y
      left = self.left + ((max_y - min_y) - (max_x - min_x)) * f
    top = self.top
    
    self.f     = f
    self.cleft = left
    self.ctop  = top
    self.min_x = min_x
    self.min_y = min_y
    
    self.flags_pos = []
    self._calllist = soyaopengl.glGenList()
    soyaopengl.glNewList(self._calllist)
    soyaopengl.glColor4f(1.0, 1.0, 1.0, 1.0)
    soyaopengl.glBegin(soyaopengl.GL_QUADS)
    for flag in flags:
      x = left + f * (flag.x - min_x)
      y = top  + f * (flag.z - min_y)
      soyaopengl.glVertex2f(x - 4.0, y - 4.0)
      soyaopengl.glVertex2f(x - 4.0, y + 4.0)
      soyaopengl.glVertex2f(x + 4.0, y + 4.0)
      soyaopengl.glVertex2f(x + 4.0, y - 4.0)
    soyaopengl.glEnd()
    
    soyaopengl.glEndList()
    
  def resize(self, parent_left, parent_top, parent_width, parent_height):
    self.left = parent_width - self.width - 40
    self.screen_width  = parent_width
    self.screen_height = parent_height
    
  def render(self):
    soyaopengl.glCallList(self._calllist)
    
#     for character in self.level.characters:
#       if isinstance(character, slune_character.Competitor):
#         x = self.cleft + self.f * (character.x - self.min_x)
#         y = self.ctop  + self.f * (character.z - self.min_y)
        
    for character in self.level.characters:
      if isinstance(character, slune_character.Competitor):
        x = self.cleft + self.f * (character.x - self.min_x)
        y = self.ctop  + self.f * (character.z - self.min_y)

        self.point.__init__(character, 0.0, 0.0, -25.0)
        self.point.convert_to(self.level)
        x2 = self.cleft + self.f * (self.point.x - self.min_x)
        y2 = self.ctop  + self.f * (self.point.z - self.min_y)

        soya.DEFAULT_MATERIAL.activate()
        soyaopengl.glBegin(soyaopengl.GL_LINES)
        soyaopengl.glVertex2f(x , y )
        soyaopengl.glVertex2f(x2, y2)
        soyaopengl.glEnd()
        
        material = soya.Material.get("head_" + character.perso_name + "_1")
        material.activate()
        soyaopengl.glEnable(soyaopengl.GL_BLEND)
        soyaopengl.glBegin(soyaopengl.GL_QUADS)
        soyaopengl.glTexCoord2f(0.0, 0.0); soyaopengl.glVertex2f(x - 20.0, y - 20.0)
        soyaopengl.glTexCoord2f(0.0, 1.0); soyaopengl.glVertex2f(x - 20.0, y + 20.0)
        soyaopengl.glTexCoord2f(1.0, 1.0); soyaopengl.glVertex2f(x + 20.0, y + 20.0)
        soyaopengl.glTexCoord2f(1.0, 0.0); soyaopengl.glVertex2f(x + 20.0, y - 20.0)
        soyaopengl.glEnd()
        soyaopengl.glDisable(soyaopengl.GL_BLEND)
        soya.DEFAULT_MATERIAL.activate()
  
        
